const { Kafka } = require("kafkajs");
const {
  SchemaRegistry,
  SchemaType,
} = require("@kafkajs/confluent-schema-registry");

// Esquema Avro, para serialización y deserialización de mensajes.
// Indica el formato del mensaje.
const schema = `
  {
    "type": "record",
    "name": "RandomTest",
    "namespace": "examples",
    "fields": [{ "type": "string", "name": "fullName" }]
  }
`;

// Credenciales de ejemplo.
// Cámbialas por el usuario y contraseña que te ha proporcionado Soporte.
const username = "username";
const password = "password";
const topic = "test-topic";

// Conexión al registro de esquemas. Se utiliza para guardar
// y recuperar todos los esquemas generados.
// La conexión está securizada. Utiliza las credenciales proporcionadas.
const registry = new SchemaRegistry({
  host: "https://schema-registry-security.nautilus.app",
  auth: {
    username,
    password,
  },
});

// Conexión con el Kafka del Datahub.
// Es obligatorio especificar los brokers con su puerto,
// protocolo SSL con el certificado previamente descargado y
// SASL con las credenciales proporcionadas.
const kafka = new Kafka({
  // Urls para los servidores de Kafka.
  brokers: [
    "public-broker1.nautilus.app:8096",
    "public-broker2.nautilus.app:8096",
    "public-broker3.nautilus.app:8096",
    "public-broker4.nautilus.app:8096",
    "public-broker5.nautilus.app:8096",
  ],
  // Llamadas securizadas. Es obligatorio usar el certificado previamente descargado.
  ssl: {
    rejectUnauthorized: true
  },
  // Autenticación para Kafka, con el método de hash SHA512.
  sasl: {
    mechanism: "scram-sha-512",
    username,
    password,
  },
});

// Código de ejecución principal.
(async () => {
  // Se registra el esquema Avro para poder producir mensajes serializados.
  // Este paso no es necesario si solo queremos consumir mensajes.
  const { id: schemaId } = await registry.register(
    {
      type: SchemaType.AVRO,
      schema
    },
    { subject: topic }
  );
  // Se crean los clientes para Kafka, un productor y un consumidor.
  const producer = kafka.producer();
  // Para consumir es recomendable crear un grupo de consumidores con un identificador único.
  const consumer = kafka.consumer({
    groupId: "f3dd7f9f-1b3c-4e47-a96c-f8018a45d175",
  });

  // En este ejemplo solo se produce un mensaje y después cerramos la conexión
  // ya que no produciremos ningún mensaje más.
  // El topic que se usa es un nombre de prueba.
  await producer.connect();
  await producer.send({
    topic,
    messages: [
      // Los datos del mensaje se serializan mediante el esquema generado anteriormente.
      {
        value: await registry.encode(schemaId, {
          type: "Whatever",
          fullName: "Just a Name",
        }),
      },
    ],
  });
  await producer.disconnect();

  // El consumidor se conecta y empieza a consumir del topic desde el inicio.
  // Consumirá todos los mensajes que sean producidos en el topic.
  await consumer.connect();
  await consumer.subscribe({ topic, fromBeginning: true });
  await consumer.run({
    // Función que se ejecuta por cada mensaje que es leido por el consumidor.
    eachMessage: async ({ topic, partition, message }) =>
      console.log(topic, partition, {
        // Deserializamos el mensaje para obtener su valor real.
        value: await registry.decode(message.value),
      }),
  });
})();
